<?php namespace EwTools\Admin\ProjectPluginPage;

use EwTools\Storage\ObjectDB;
use EwTools\Support\Arr;
use EwTools\Support\Collection;
use EwTools\Support\Str;

/**
 * ComponentsPage
 *
 * @link    http://infomaker.se
 * @package EwTools\Admin\ProjectPluginPage
 * @since   EwTools\Admin\ProjectPluginPage\ComponentsPage 0.1
 */
abstract class ComponentsPage {
    
    /**
     * Option to store if plugins should be force saved
     *
     * @var string
     */
    protected static $force_save_option = 'force_everyware_plugins';
    
    /**
     * @var string
     */
    protected $text_domain = 'ewtools_textdomain';
    
    /**
     * Contains the widgets available for management
     *
     * @var array
     */
    protected $available_components = null;
    
    /**
     * Contains a list of active plugins without the components
     *
     * @var array
     */
    protected $filtered_plugins = null;
    
    /**
     * Contains a list of active plugins combined with the components
     *
     * @var null
     */
    protected $combined_plugins = null;
    
    /**
     * Contains setting of whether the components should force add them selves to the list of active plugins before save
     *
     * @var bool
     */
    protected $force_save = false;
    
    /**
     * Active plugins options
     *
     * @var string
     */
    protected static $option_name;
    
    /**
     * @var string
     */
    protected static $slug;
    
    /**
     * @var string
     */
    protected static $title;
    
    /**
     * Contains the path to the folder where the specified components age stored
     *
     * @var
     */
    private $components_path;
    
    /**
     * Contains the update state of the components-list
     *
     * @var bool
     */
    protected $updated = false;
    
    /**
     * ComponentsPage constructor.
     *
     * @since 0.1
     *
     * @param $components_path
     */
    public function __construct( $components_path ) {
        
        $this->components_path = $components_path;
        $this->shouldForceSave(ObjectDB::get( static::$force_save_option ));
        /**
         * Hiding our own plugins for WordPress so they don't deactivate them.
         */
        add_filter( 'option_active_plugins', [ $this, 'hideComponents' ], 10, 1 );
        
        /**
         * We are hooking in and adding our own plugins before saving the original plugins
         * A must do to first hide our plugins and the hook this on otherwise
         * WordPress will say our plugins is invalid and deactivate them.
         */
        add_filter( 'pre_update_option_active_plugins', [ $this, 'forceSaveComponents' ], 10, 1 );
    }
    // Filters/actions:
    // ======================================================
    
    /**
     * Hide plugins if we are on plugins.php so WordPress
     * don't deactivate them.
     *
     * @param $plugins
     *
     * @return array
     */
    public function hideComponents( $plugins = null ) {
        $plugins = $plugins ?: static::allPlugins();
        return $this->onWpPlugins() ? $this->getFilteredPlugins( $plugins ) : $plugins;
    }
    
    /**
     * To prepend our plugins to WordPress original update plugins
     *
     * @param $plugins
     *
     * @return mixed
     */
    public function forceSaveComponents( $plugins = null) {
        $plugins = $plugins ?: static::allPlugins();
        return $this->shouldForceSave() ? $this->getCombinedPlugins( $plugins ) : $plugins;
    }
    
    // Public functions:
    // ======================================================
    
    /**
     * Retrieve the slug to be use to identify the page
     *
     * @since 0.1
     * @return string
     */
    public function getSlug() {
        return Str::slug( static::$slug );
    }
    
    /**
     * Retrieve the title of the page to be used in the menu
     *
     * @since 0.1
     * @return string
     */
    public function getTitle() {
        return __( Str::title( static::$title ), $this->text_domain );
    }
    
    /**
     * Retrieve stored components from the options table and try to activate them.
     * Restores list after activation
     *
     * @since 0.1
     * @return bool
     */
    public function activateComponents() {
        $components = $this->activeComponents();
        $components = Arr::hasStringKeys( $components ) ? array_keys( $components ) : $components;
        foreach ( $components as $component ) {
            if( ! $this->activateComponent( $component ) ) {
                unset( $components[ $component ] );
            }
        }
        $this->shouldForceSave(true);
        return $this->storeComponents( $components );
    }
    
    /**
     * Activate component as a Wordpress plugin
     *
     * @param $component
     *
     * @return bool
     */
    public function activateComponent( $component ) {
        $file = WP_PLUGIN_DIR . '/' . $component;
        
        // Check so the plugin is available
        if( is_readable( $file ) ) {
            
            // Include file so activate can be triggered.
            include_once $file;
            
            do_action( 'activate_plugin', $component );
            do_action( 'activate_' . $component );
            do_action( 'activated_plugin', $component );
            
            return true;
        }
        
        return false;
    }
    
    /**
     * Use {add_submenu_page} to add the page to the plugin menu.
     *
     * @param string $parent_slug
     *
     * @since 0.1
     * @return string the menu hook_suffix
     */
    public function addToMenu( $parent_slug ) {
        // Widgets
        $hook = add_submenu_page( $parent_slug, $this->getTitle(), $this->getTitle(), 'manage_options', Str::slug( $parent_slug . "-" . $this->getTitle() ), [
            $this,
            'renderPage'
        ] );
        
        add_action( "load-{$hook}", [ $this, 'onPageLoad' ] );
    }
    
    /**
     * Retrieve a list of components from the specified folder
     * Used for fetching "Widgets" and "Plugins"
     *
     * @since 0.1
     * @return array
     */
    public function availableComponents() {
        
        if( ! $this->available_components ) {
            $components = get_plugins( '/' . $this->components_path );
            
            foreach ( $components as $file => $component ) {
                $component_path                  = "{$this->components_path}/{$file}";
                $components[ $file ][ 'Active' ] = in_array( $component_path, static::allPlugins() );
                $components[ $file ][ 'Path' ]   = $component_path;
            }
            $this->available_components = $components;
        }
        
        return $this->available_components;
    }
    
    /**
     * Retrieve activated components from its options table
     *
     * @param mixed $default
     *
     * @since 0.1
     * @return mixed
     */
    public function activeComponents( $default = [] ) {
        return ObjectDB::get( static::$option_name, $default );
    }
    
    /**
     * Deactivate all active components
     *
     * @since 0.1
     * @return void
     */
    public function deactivateComponents() {
        $components = $this->activeComponents();
        $components = Arr::hasStringKeys( $components ) ? array_keys( $components ) : $components;
        foreach ( $components as $component ) {
            $this->deactivateComponent( $component );
        }
        $this->shouldForceSave(false);
    }
    
    /**
     * Deactivate component from wordpress plugins
     *
     * @param $component
     *
     * @since 0.1
     * @return void
     */
    public function deactivateComponent( $component ) {
        do_action( 'deactivate_plugin', $component );
        do_action( "deactivate_{$component}");
        do_action( 'deactivated_plugin', $component );
    }
    
    /**
     * Determine if a widget is available for management via Wordpress
     *
     * @param string $component
     *
     * @since 0.1
     * @return void
     */
    public function exist( $component ) {
        return in_array( $this->getFileName( $component ), array_keys( $this->availableComponents() ) );
    }
    
    /**
     * Return the filename from an Everyware component
     *
     * @param $component
     *
     * @since 0.1
     * @return string
     */
    public function getFileName( $component ) {
        if( Str::startsWith( $component, $this->components_path ) ) {
            return Str::substr( $component, Str::length( Str::finish( $this->components_path, '/' ) ) );
        }
        
        return $component;
    }
    
    /**
     * Retrieve a merged list of active plugins and Everyware components
     *
     * @param array $plugins
     *
     * @since 0.1
     * @return array
     */
    public function getCombinedPlugins( $plugins = null ) {
        if ( ! $this->combined_plugins ) {
            $plugins = $plugins ?: static::allPlugins();
    
            $active_components = $this->activeComponents();
            $active_components = Arr::hasStringKeys( $active_components ) ? array_keys( $active_components ) : $active_components;
    
            $this->combined_plugins = array_unique( array_merge( $plugins, $active_components ) );
        }
        
        return $this->combined_plugins;
    }
    
    /**
     * Retrieve a list of plugins from Wordpress options table and filter out the components
     *
     * @since 0.1
     * @return array
     */
    public function getFilteredPlugins($plugins = null) {
        if ( ! $this->filtered_plugins ) {
            $plugins = $plugins ?: static::allPlugins();
        
            $this->filtered_plugins = Arr::where( $plugins, function ( $key, $plugin ) {
                return Str::not( Str::finish( $this->components_path, '/*' ), $plugin );
            } );
        }
        
        return $this->filtered_plugins;
    }
    
    /**
     * Check if a value is a Component added by Everyware
     *
     * @param string $component
     *
     * @since 0.1
     * @return bool
     */
    public function isComponent( $component ) {
        return Str::is( Str::finish( $this->components_path, '/*' ), $component );
    }
    
    /**
     * Function that fires on page load
     *
     * @since 0.1
     * @return mixed
     */
    public function onPageLoad() {
        $this->updated = $this->onComponentsUpdate() ? $this->updateActiveState() : false;
        
        if ( $this->updated ) {
            $active_components = $this->activeComponents();
            $active_components = Arr::hasStringKeys( $active_components ) ? array_keys( $active_components ) : $active_components;
            // Update active plugins with the others
            return ObjectDB::update( 'active_plugins', array_merge( $active_components, $this->getFilteredPlugins() ) );
        }
    }
    
    /**
     * Run through activated widgets and make sure they exist.
     * Update options table if diff occurs or force update with parameter
     *
     * @param bool $update force update
     *
     * @since 0.1
     * @return array the updated list of widgets
     */
    public function updateActiveComponents( $update = false ) {
        if( ( $active_components = $this->activeComponents(false) ) !== false ) {
            foreach ( array_keys( $active_components ) as $component ) {
                if( ! $this->exist( $component ) ) {
                    $update = true;
                    unset( $active_components[ $component ] );
                }
            }
            if( $update ) {
                $this->storeComponents( $active_components );
            }
            
            return Arr::hasStringKeys( $active_components ) ? array_keys( $active_components ) : $active_components;
        }
        
        return [];
    }
    
    /**
     * Activate/Deactivate components according to the new list
     *
     * @since 0.1
     * @return bool
     */
    public function updateActiveState() {
        $active_components = $this->activeComponents();
        $new_components = $this->getSubmittedComponents();
        // Deactivate components that isn't in the new active list
        foreach ( array_keys( $active_components ) as $component ) {
            if( ! in_array( $component, array_keys( $new_components ) ) ) {
                $this->deactivateComponent( $component );
            }
        }
        
        // Activate components that isn't in the old active list
        foreach ( array_keys( $new_components ) as $component ) {
            if( ! in_array( $component, array_keys( $active_components )) ) {
                
                // Check if widgets exist if so activate it
                if( ! $this->activateComponent( $component ) ) {
                    unset( $new_components[ $component ] );
                }
            }
        }
        
        return $this->storeComponents($new_components);
    }
    
    // Protected Functions:
    // ======================================================
    
    /**
     * Retrieve component page specific data with added data to handle management of component-list
     *
     * @param array $custom_data
     *
     * @since 0.1
     * @return array
     */
    protected function getComponentsPageData( $custom_data = [] ) {
        $current_list_status_filter = $this->getCurrentStatusFilter();
        $view_data                  = array_replace_recursive( [
            'component_type'    => $this->getTitle(),
            'description'       => __( 'From here you can manage your Everyware components for the site.', $this->text_domain ),
            'updated'           => $this->onComponentsUpdate(),
            'plugin_status'     => $current_list_status_filter,
            'plugin_status_url' => [
                'all'      => add_query_arg( [ 'plugin_status' => 'all' ] ),
                'active'   => add_query_arg( [ 'plugin_status' => 'active' ] ),
                'inactive' => add_query_arg( [ 'plugin_status' => 'inactive' ] )
            ],
            'form_action'       => add_query_arg( [ 'settings-updated' => 'true' ] ),
            'text_domain'       => $this->text_domain,
            'components'        => $this->availableComponents(),
            'active_components' => []
        ], $custom_data );
        
        $components = new Collection( $view_data[ 'components' ] );
        
        $view_data[ 'total_count' ]    = $components->count();
        $view_data[ 'active_count' ]   = $components->where( 'Active', true )->count();
        $view_data[ 'inactive_count' ] = $components->where( 'Active', false )->count();
        
        return $view_data;
    }
    
    /**
     * Retrieve the current status-filter of the components-list
     *
     * @since 0.1
     * @return string
     */
    protected function getCurrentStatusFilter() {
        return isset( $_GET[ 'plugin_status' ] ) ? $_GET[ 'plugin_status' ] : 'all';
    }
    
    /**
     * Check if components management page have posted changes
     *
     * @since 0.1
     * @return bool
     */
    protected function onComponentsUpdate() {
        return isset( $_POST[ 'everyware-component-submit' ] );
    }
    
    /**
     * Check if on plugins.php aka Wordpress plugins management
     *
     * @since 0.1
     * @return bool
     */
    protected function onWpPlugins() {
        global $pagenow;
        
        return $pagenow === 'plugins.php';
    }
    
    /**
     * Update specific Components in options table
     *
     * @param array $components
     *
     * @since 0.1
     * @return bool
     */
    protected function storeComponents( $components = [] ) {
        return ObjectDB::update( static::$option_name, $components );
    }
    
    /**
     * Retrieve new list of components posted by the admin page
     *
     * @since 0.1
     * @return array
     */
    protected function getSubmittedComponents() {
        return isset( $_POST[ 'active_components' ] ) && is_array( $_POST[ 'active_components' ] ) ? $_POST[ 'active_components' ] : [];
    }
    
    /**
     * Determine if plugins and widgets should be force-saved as "active_plugins" in Wordpress
     * May be changed from outside users
     *
     * @param bool $value
     *
     * @since 0.1
     * @return bool
     */
    protected function shouldForceSave($value= null) {
        if ( is_bool($value) ) {
            ObjectDB::update( static::$force_save_option, $value );
            $this->force_save = $value;
        }
        return $this->onWpPlugins() && $this->force_save;
    }
    
    // Abstract Functions: Functions that must be implemented by child-classes
    // ======================================================
    
    /**
     * Function for outputting the admin page of the components from the menu
     *
     * @since 0.1
     * @return mixed
     */
    abstract public function renderPage();
    
    // Static Functions:
    // ======================================================
    /**
     * Retrieve all "Plugins" activated in Wordpress from the options table
     *
     * @since 0.1
     * @return mixed
     */
    protected static function allPlugins() {
        return ObjectDB::get( 'active_plugins', [] );
    }
}