<?php namespace EwTools\Support;

/**
 * Utilities
 *
 * @link    http://infomaker.se
 * @package EwTools\Support
 * @since   EwTools\Support\Utilities 0.1
 */
class Utilities {
    
    /**
     * Determine if we are in a production environment
     *
     * @since 0.1
     * @return bool
     */
    public static function isProd() {
        return getenv( 'ENVIRONMENT' ) === 'prod';
    }
    
    /**
     * Determine if we are in a development environment
     *
     * @since 0.1
     * @return bool
     */
    public static function isDev() {
        $env = getenv( 'ENVIRONMENT' );
        return $env !== 'prod' && $env !== 'stage';
    }
    
    /**
     * Function for fetching the correct js uri depending on if you are using the base- or child-theme's javascript
     *
     * @param string $file_name
     *
     * @since 0.1
     * @return string
     */
    public static function getCorrectJsUrl( $file_name = '' ) {
        return Str::finish( static::getStaticTemplateUri(), "/assets/js/{$file_name}" );
    }
    
    /**
     * Function to retrieve current host
     *
     * @since 0.1
     * @return mixed
     */
    public static function getCurrentHost() {
        return parse_url( site_url(), PHP_URL_HOST );
    }
    
    /**
     * Return the absolute path of the requested plugin
     *
     * @param string $plugin_dir name of the plugin dir
     *
     * @since 0.1
     * @return string
     */
    public static function pluginPath( $plugin_dir ) {
        return ABSPATH . 'wp-content/plugins/' . ltrim( $plugin_dir, '/' );
    }
    
    /**
     * Return path to plugin uri
     *
     * @since 0.1
     *
     * @param string $plugin_dir name of the plugin
     *
     * @return mixed|string
     */
    public static function getStaticPluginUri( $plugin_dir = '' ) {
        return static::getCdnUri( plugins_url( $plugin_dir ) );
    }
    
    /**
     * Return theme uri for static assets, if prod return with cloud-front.
     *
     * @since 0.1
     * @return string
     */
    public static function getStaticThemeUri() {
        
        return static::getCdnUri( get_stylesheet_directory_uri() );
    }
    
    /**
     * Return template uri for static assets, if prod return with cloudfront.
     *
     * @since 0.1
     * @return string
     */
    public static function getStaticTemplateUri() {
        return static::getCdnUri( get_template_directory_uri() );
    }
    
    /**
     * Retrieve the cdn version of chosen uri, if prod return with cloudf-ront
     *
     * @param string $uri
     *
     * @since 0.1
     * @return string
     */
    public static function getCdnUri( $uri = '' ) {
        $cdn_uri = getenv( 'CF_STATIC' );
        $uri = empty( $uri ) ? get_stylesheet_directory_uri() : $uri;
        
        if( null !== $cdn_uri && static::isProd() ) {
            return substr_replace( $uri, $cdn_uri, 0, strpos( $uri, '/wp-content/' ) );
        }
        
        return $uri;
    }
    
    /**
     * Retrieve the cdn uri for theme files, if prod return with cloudfront
     *
     * @param $file_path
     *
     * @since 0.1
     * @return string
     */
    public static function getCdnFile( $file_path ) {
        return Str::finish( static::getCdnUri(), '/' ) . ltrim( $file_path, '/\\' );
    }
}
